<?php

/*
	characters are the living, breathing entities that interact with the game world. They are inextricably linked to their
	accounts, and are the primary means by which the character interacts with the game world. Separating the character from
	the account allows for multiple characters to be associated with a single account, and to prevent concurrency issues
	when performing auth checks on the database.

	When creating a character, we want to init all of the related data tables; wallets, inventory, bank, etc.

	When retrieving a character, we will get the tables as-needed, to prevent allocating more memory than we need.
*/

const currently = [
	0 => 'Exploring',
	1 => 'In Town',
	2 => 'In Combat',
	4 => 'In Shop',
	5 => 'In Inn'
];

/**
 * Create a character. Only a user ID and a name are required. All other fields are optional. Pass a key-value array
 * of overrides to set additional fields. A character's name must be unique, but this function does not check for
 * that. Returns the created character's ID.
 */
function char_create($user_id, $name, $overrides = []): int
{
	// Prep the data and merge in any overrides
	$data = ['user_id' => $user_id, 'name' => $name];
	if (!empty($overrides)) $data = array_merge($data, $overrides);

	// Prep the fields for the query
	$k = array_keys($data);
	$f = implode(', ', $k);
	$v = implode(', ', array_map(fn($x) => ":$x", $k));

	// Create the character!
	if (db_query(db_live(), "INSERT INTO characters ($f) VALUES ($v)", $data) === false) {
		// @TODO: Log this error
		throw new Exception('Failed to create character. (cc)');
	}

	// Get the character ID
	return db_live()->lastInsertRowID();
}

/**
 * Create a character's location record. A character's location is where they are in the game world. A character can only be
 * in one location at a time. Can define a starting location for the character. Default state is 'Exploring'.
 */
function char_location_create($char_id, $x = 0, $y = 0, $currently = 0)
{
	if (db_query(db_live(), "INSERT INTO char_locations (char_id, x, y, currently) VALUES (:p, :x, :y, :c)", [
		':p' => $char_id,
		':x' => $x,
		':y' => $y,
		':c' => $currently
	]) === false) {
		throw new Exception('Failed to create character location. (clc)');
	}
}

/**
 * Create the character's gear table. A character's gear is where they store their equipped items.
 * @TODO: implement initial gear
 */
function char_gear_create($char_id, $initialGear = [])
{
	if (db_query(db_live(), "INSERT INTO char_gear (char_id) VALUES (:p)", [':p' => $char_id]) === false) {
		throw new Exception('Failed to create character gear. (cgc)');
	}
}

/**
 * Get a charcter by their ID. Returns the character's data as an associative array, or false if not found.
 */
function char_find($char_id)
{
	$char = db_query(db_live(), "SELECT * FROM characters WHERE id = :id", [':id' => $char_id])->fetchArray(SQLITE3_ASSOC);
	return $char === false ? false : $char;
}

/**
 * Count the number of characters associated with an account ID.
 */
function char_count($user_id): int
{
	$count = db_query(db_live(), "SELECT COUNT(*) FROM characters WHERE user_id = :u", [':u' => $user_id])->fetchArray(SQLITE3_NUM);
	if ($count === false) throw new Exception('Failed to count characters. (cc)');
	return (int) $count[0];
}

/**
 * Get a an array of id => [name, level] for all characters associated with an account ID.
 */
function char_list($user_id)
{
	$stmt = db_query(db_live(), "SELECT id, name, level FROM characters WHERE user_id = :u", [':u' => $user_id]);
	if ($stmt === false) throw new Exception('Failed to list characters. (cl)');

	$characters = [];
	while ($row = $stmt->fetchArray(SQLITE3_ASSOC)) {
		$characters[$row['id']] = ['name' => $row['name'], 'level' => $row['level']];
	}

	return $characters;
}

/**
 * Get a character's location info by their character ID. Returns the location's data as an associative array.
 */
function char_get_location($char_id)
{
	// Get the location
	$location = db_query(db_live(), "SELECT * FROM char_locations WHERE char_id = :p", [':p' => $char_id])->fetchArray(SQLITE3_ASSOC);
	if ($location === false) throw new Exception('Location not found. (cgl)');
	return $location;
}

/**
 * See if a character name exists.
 */
function char_name_exists($name)
{
	return db_exists(db_live(), 'characters', 'name', $name);
}

/**
 * Checks whether a character exists at a certain ID.
 */
function char_exists($char_id)
{
	return db_exists(db_live(), 'characters', 'id', $char_id);
}

/**
 * See if the given character belongs to the given user. Returns false if the character does not belong to the user,
 * or if the character does not exist. Returns true if the character belongs to the user. Generally this function
 * shouldn't return false, as it should be called after the character's existence is confirmed. If it does return false,
 * it is likely due to user interference.
 */
function char_belongs_to_user($char_id, $user_id)
{
	$char = db_query(
		db_live(),
		"SELECT 1 FROM characters WHERE id = :p AND user_id = :u",
		[':p' => $char_id, ':u' => $user_id]
	)->fetchArray(SQLITE3_ASSOC);
	return $char !== false;
}

/**
 * Delete a character by their ID. This will delete all associated data tables as well.
 */
function char_delete($char_id)
{
	// Delete the character
	if (db_query(db_live(), "DELETE FROM characters WHERE id = :p", [':p' => $char_id]) === false) {
		throw new Exception('Failed to delete character. (cd)');
	}

	// Get item IDs from the character's inventory
	$items = db_query(db_live(), "SELECT item_id FROM char_inventory WHERE char_id = :p", [':p' => $char_id]);
	// delete the character's inventory and items
	while ($row = $items->fetchArray(SQLITE3_ASSOC)) {
		if (db_query(db_live(), "DELETE FROM char_inventory WHERE char_id = :c", [':c' => $char_id]) === false) {
			throw new Exception('Failed to delete character inventory. (cd)');
		}

		if (db_query(db_live(), "DELETE FROM items WHERE id = :p", [':p' => $row['id']]) === false) {
			throw new Exception('Failed to delete character item slots. (cd)');
		}
	}

	// Delete the character's location
	if (db_query(db_live(), "DELETE FROM char_locations WHERE char_id = :p", [':p' => $char_id]) === false) {
		throw new Exception('Failed to delete character location. (cd)');
	}

	// Delete the character's gear
	if (db_query(db_live(), "DELETE FROM char_gear WHERE char_id = :p", [':p' => $char_id]) === false) {
		throw new Exception('Failed to delete character gear. (cd)');
	}

	// Delete the character's bank
	if (db_query(db_live(), "DELETE FROM char_bank WHERE char_id = :p", [':p' => $char_id]) === false) {
		throw new Exception('Failed to delete character bank. (cd)');
	}

	// Delete character's banked items
	if (db_query(db_live(), "DELETE FROM char_banked_items WHERE char_id = :p", [':p' => $char_id]) === false) {
		throw new Exception('Failed to delete character bank items. (cd)');
	}

	// Delete the user's guild membership
	if (db_query(db_live(), "DELETE FROM guild_members WHERE char_id = :p", [':p' => $char_id]) === false) {
		throw new Exception('Failed to delete character guild membership. (cd)');
	}

	// if the character was a guild leader, hand leadership to the next highest ranking member
	$guild = db_query(db_live(), "SELECT id FROM guilds WHERE leader_id = :p", [':p' => $char_id])->fetchArray(SQLITE3_ASSOC);
	if ($guild !== false) {
		$members = db_query(db_live(), "SELECT char_id FROM guild_members WHERE guild_id = :p ORDER BY rank DESC", [':p' => $guild['id']]);
		$newLeader = $members->fetchArray(SQLITE3_ASSOC);
		if ($newLeader !== false) {
			db_query(db_live(), "UPDATE guilds SET leader_id = :p WHERE id = :g", [':p' => $newLeader['char_id'], ':g' => $guild['id']]);
		}
	}

	// Get a list of all pve fight IDs.
	$pve = db_query(db_fights(), "SELECT id FROM pve WHERE char_id = :p", [':p' => $char_id]);
	// Get a list of all pvp fight IDs.
	$pvp = db_query(db_fights(), "SELECT id FROM pvp WHERE char1_id = :p OR char2_id = :p", [':p' => $char_id]);

	// Delete all pve fights
	while ($row = $pve->fetchArray(SQLITE3_ASSOC)) {
		if (db_query(db_fights(), "DELETE FROM pve WHERE id = :p", [':p' => $row['id']]) === false) {
			throw new Exception('Failed to delete pve fight. (cd)');
		}

		if (db_query(db_fights(), "DELETE FROM pve_logs WHERE fight_id = :p", [':p' => $row['id']]) === false) {
			throw new Exception('Failed to delete pve fight logs. (cd)');
		}
	}

	// Delete all pvp fights
	while ($row = $pvp->fetchArray(SQLITE3_ASSOC)) {
		if (db_query(db_fights(), "DELETE FROM pvp WHERE id = :p", [':p' => $row['id']]) === false) {
			throw new Exception('Failed to delete pvp fight. (cd)');
		}

		if (db_query(db_fights(), "DELETE FROM pvp_logs WHERE fight_id = :p", [':p' => $row['id']]) === false) {
			throw new Exception('Failed to delete pvp fight logs. (cd)');
		}
	}
}
